// fft.c, created by James Rishe and Andrew Melkonian
// using examples from team Nortsam

#include "xbasic_types.h"
#include "xio.h"
#include "coeff1024.c"  // coefficients must be calculated for FFT size
#include "fft_settings.h"

//extern Xuint32 coeff[];


void separate(Xint32* adata, Xint16 *dataL ) {
  int i,j;
  Xint16* aiter, *aend;
  Xint16* diter, *dend;

  aiter = adata+1;
  diter = dataL+1;
  aend = adata+FFT_SIZE;
  dend = dataL+(FFT_SIZE<<1);
  
  for(i = 1,j=1 ; i <= FFT_SIZE;++i, j+=2 ){
    dataL[j] = (Xint16)((adata[i] & 0xFFFF0000) >> 16);
    //    dataL[j] <<=1; //amplify
    dataL[j+1] = 0x0000;
    //dataR[i] = (Xint16)(adata[i] & 0x0000FFFF);
  }
}

/**
 * takes buffer data in dataL, in form 0xLLLLRRRR
 * separates L and R data and performs bit reversal of indices
 * returns two arrays, each ready to be FFT-ized
 * ---note: as with the FFT algorithm, this function will never access idx[0]
 * fills all imaginary (even) indices with 0x0000.
 */
void bitReverse( Xint16* data ){
  int i, j, n, m;
  Xint16 temp;

  n = FFT_SIZE << 1;
  j = 1;
  
  for ( i=1; i <= n; i+=2 ) {
    if ( j > i ) {

      temp = data[i];
      data[i] = data[j];
      data[j] = temp;

      /*  Not necessary because these are
	  zero-valued
      temp = data[i+1];
      data[i+1] = data[j+1];
      data[j+1] = temp; */

    }
    
    m = FFT_SIZE;
    while ( (m >= 2) && (j > m) ) {
      j -= m;
      m >>= 1;
    }

    j += m;
  }

}

// takes an array of size (FFT_SIZE * 2), of 16-bit values
// this function WILL NEVER access array index 0 (very important-no seg faults)
void doFFT( Xint16 *data ) {
  //  int istep, mmax, j, i, m, n, coeff_ofs, z;
  Xint32 istep, mmax, j, i, m, n;
  Xuint32* coeffp;

  Xint16* tempRI;
  Xuint32 temp=0;
  Xint16 tempr;
  Xint16 tempi;

  coeffp = coeff;

  n = FFT_SIZE << 1;
  mmax = 2;
  while ( n > mmax ) { // this, the outer loop, gets executed log2(FFT_SIZE) times.
    istep = mmax << 1;
    //    print("");
    for ( m=1; m<mmax; m+=2 ) {
      for ( i=m; i<=n; i+=istep ) {
	j = i + mmax;

	// The Danielson-Lanczos formula, embedded systems
	// style
	//Write data to the multiplier (no imaginary part)
	XIo_Out32( 0x0FEF0008, ((Xuint32)data[j])<<16);
	//Write coefficient to the multiplier
	XIo_Out32( 0x0FEF000C, *coeffp );
	temp = XIo_In32( 0x0FEF0000 );
	
	//	tempr = (Xint16)(temp >> 16);
	//	tempi = (Xint16)(temp & 0x0000FFFF);
	tempRI = (Xint16*)(&temp);
	
	data[j] = data[i] - tempRI[0];
	data[j+1] = data[i+1] - (tempRI[1]);
	data[i] += tempRI[0];
	data[i+1] += tempRI[1];
	/*
	data[j] = data[i] - (Xint16)(temp>>16);
	data[j+i] = data[i+1] - temp&0x0000FFFF;
	data[i] += (Xint16)(temp>>16);
	data[i+1] += temp&0x0000FFFF;
	*/
      }
      coeffp++;
    }
    mmax = istep;
    /*    putnum(n);
    print("$");
    putnum(mmax);
    print("$\n\r");*/


  }
  //  print("FFT DONE \n\r");
  //print("Going back now");
}

