// graphics.c, created by James Rishe, Andrew Melkonian
//   Narashiman Chakravarthy
// Using some functionality from Nortsam

#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"
#include "fft_settings.h"

#define W 640
#define H 480
#define VGA_START 0x00800000
#define X_START 40
#define Y_START 240

#define RED 0xE0
#define GREEN 0x1C
#define BLUE 0x03
#define WHITE RED | GREEN | BLUE
#define BLACK 0x00

#define MAX_PIX 200

#define TRUE 1
#define FALSE 0

/*#define CHAR(r,c) \
  ([(c) + ((r) << 6) + ((r) << 4)])*/
#define CHAR(r,c) \
   (((unsigned char *)(VGA_START))[(c) + ((r) << 6) + ((r) << 4)])

void put_string(char *string, int row, int column)
{
  char *p = &(CHAR(row, column));
  while (*p++ = *string++)
    ;
}


Xuint16 correspond[] = {
     0,    1,    2,    3,    4,    5,    6,    7,    8,    9, 
    10,   11,   13,   16,   20,   25,   31,   39,   49,   61, 
    77,   97,  122,  153,  194,  244,  306,  387,  487,  612, 
   773,  973};
/********************************************************************/
Xuint16 range[] = { 
  //    1     2     3      4      5      6      7      8      9
  0,    0,    1,    1,     1,     2,     2,     2,     2,     2,
  2,    3,    3,    3,     3,     3,     3,     3,     3,     3,
  3,    3,    3,    3,     4,     4,     4,     4,     4,     4,
  4,    4,    4,    4,     4,     4,     4,     4,     4,     4,
  4,    4,    4,    4,     4,     4,     4,     4,     5,     5,
  5,    5,    5,    5,     5,     5,     5,     5,     5,     5,
  5,    5,    5,    5,     5,     5,     5,     5,     5,     5,
  5,    5,    5,    5,     5,     5,     5,     5,     5,     5,
  5,    5,    5,    5,     5,     5,     5,     5,     5,     5,
  5,    5,    5,    5,     5,     5,     6,     6,     6,     6,
  6,    6,    6,    6,     6,     6,     6,     6,     6,     6,
  6,    6,    6,    6,     6,     6,     6,     6,     6,     6,
  6,    6,    6,    6,     6,     6,     6,     6,     6,     6,
  6,    6,    6,    6,     6,     6,     6,     6,     6,     6,
  6,    6,    6,    6,     6,     6,     6,     6,     6,     6,
  6,    6,    6,    6,     6,     6,     6,     6,     6,     6,
  6,    6,    6,    6,     6,     6,     6,     6,     6,     6,
};

int lastmax;



Xuint16 calcMagnitude( Xuint16 datar, Xuint16 datai ) {
  Xint16 *tempRI;
  Xuint32 result;
  short high = 182, low = 0, middle;
  unsigned char less = 0;
  short temp;

  // calculate magnitudes of buckets corresponding to positive frequencies
  // hw address 0x0fef_0014 squares real and imag.
  
  XIo_Out32( 0x0FEF0014, (Xuint32)(datar)<<16 | (Xuint32)datai  );
  result = XIo_In32( 0x0FEF0000 );
  
  tempRI = (Xuint16*)(&result);
  result = (Xuint32)(tempRI[0]) + (Xuint32)(tempRI[1]);

  // saturate, don't overflow
  if (result > 32767)
    result = 0;
  return result;
}

const int startrow = 13;
const int col = 25;
const int inc = 3;

void setup_graphics(){
  int sr = startrow;
  put_string("The B^ B# Pitch Detector", 2, 3);
  put_string("Very High pitch", sr, col);
  put_string("High pitch", sr+=inc, col);
  put_string("Mid-high pitch", sr+=inc, col);
  put_string("Mid pitch", sr+=inc, col);
  put_string("Mid-low pitch", sr+=inc, col);
  put_string("Low pitch", sr+=inc, col);
  put_string("No sound", sr+=inc, col);

}

/* Wipe the screen in any places that are dynamically
   written
*/
void wipe(){
  int c = col - 2;
  int i;
  int incx5 = (inc << 2 + inc);
  for(i = startrow; i < startrow+incx5; i+= inc){
    CHAR(i, c) = ' ';
  }
}

void showgraphics(Xint16 ind){
  int i;
  int a = 6-ind;
  int nincs = inc*a;
  wipe();
  CHAR(startrow+nincs, col-2) = 13;

}

int tally[] = {0, 0, 0, 0, 0, 0, 0};
int iternum = 0;
void visualize(Xint16* data){
  Xint16 maxmag, maxind, k, i;
  maxmag = maxind = 0;

  for(k = 1; k < 340; k+=2){
    if(calcMagnitude(data[k], data[k+1]) > maxmag){
      maxind = k;
      maxmag = calcMagnitude(data[k], data[k+1]);
    }
  }
  
  tally[range[maxind>>1]]++;
  iternum++;
  if(iternum == 10){
    maxind = 0;
    maxmag = 1;
    for(i = 1; i < 7; i++){
      if(tally[i] > maxmag){
	maxind = i;
	maxmag = tally[i];
      }
      tally[i] = 0;
    }
    showgraphics(maxind);
    iternum = 0;
  }
}
 	       

